# -*- coding: utf-8 -*-

import re
from typing import Dict, Iterator, NamedTuple, Type, TypeVar, Union, overload

__all__ = ["countries"]

StrOrInt = Union[str, int]
_D = TypeVar("_D")


class Country(NamedTuple):
    name: str
    apolitical_name: str
    numeric: str
    alpha2: str
    alpha3: str


_records = [
    Country('가나', 'Ghana', '288', 'GH', 'GHA'),
    Country('가봉', 'Gabon', '266', 'GA', 'GAB'),
    Country('가이아나', 'Guyana', '328', 'GY', 'GUY'),
    Country('감비아', 'Gambia', '270', 'GM', 'GMB'),
    Country('건지', 'Bailiwick of Guernsey', '831', 'GG', 'GGY'),
    Country('경제협력개발기구', '', '   ', 'XD', '   '),
    Country('과들루프', 'Guadeloupe', '312', 'GP', 'GLP'),
    Country('과테말라', 'Guatemala', '320', 'GT', 'GTM'),
    Country('괌', 'Guam', '316', 'GU', 'GUM'),
    Country('교황청', 'Vatican', '336', 'VA', 'VAT'),
    Country('국제연합', '', '   ', 'UN', '   '),
    Country('그레나다', 'Grenada', '308', 'GD', 'GRD'),
    Country('그리스', 'Greece', '300', 'GR', 'GRC'),
    Country('그린란드', 'Greenland', '304', 'GL', 'GRL'),
    Country('기니', 'Guinea', '324', 'GN', 'GIN'),
    Country('기니비사우', 'Guinea-Bissau', '624', 'GW', 'GNB'),
    Country('나미비아', 'Namibia', '516', 'NA', 'NAM'),
    Country('나우루', 'Nauru', '520', 'NR', 'NRU'),
    Country('나이지리아', 'Nigeria', '566', 'NG', 'NGA'),
    Country('남수단', 'South Sudan', '728', 'SS', 'SSD'),
    Country('남아프리카공화국', 'South Africa', '710', 'ZA', 'ZAF'),
    Country('네덜란드', 'Netherlands', '528', 'NL', 'NLD'),
    Country('네덜란드령 안틸레스', 'Netherlands Antilles', '530', 'AN', 'ANT'),
    Country('네팔연방', 'Nepal', '524', 'NP', 'NPL'),
    Country('노르웨이', 'Norway', '578', 'NO', 'NOR'),
    Country('뉴 칼레도니아', 'New Caledonia', '540', 'NC', 'NCL'),
    Country('뉴질랜드', 'New Zealand', '554', 'NZ', 'NZL'),
    Country('니우에', 'Niue', '570', 'NU', 'NIU'),
    Country('니제르', 'Niger', '562', 'NE', 'NER'),
    Country('니카라과', 'Nicaragua', '558', 'NI', 'NIC'),
    Country('대만', 'Taiwan', '158', 'TW', 'TWN'),
    Country('대한민국', 'Korea', '410', 'KR', 'KOR'),
    Country('덴마크', 'Denmark', '208', 'DK', 'DNK'),
    Country('도미니카공화국', 'Dominican Republic', '214', 'DO', 'DOM'),
    Country('도미니카연방', 'Dominica', '212', 'DM', 'DMA'),
    Country('독일', 'Germany', '276', 'DE', 'DEU'),
    Country('동티모르', 'Timor-Leste', '626', 'TL', 'TLS'),
    Country('라오스', 'Laos', '418', 'LA', 'LAO'),
    Country('라이베리아', 'Liberia', '430', 'LR', 'LBR'),
    Country('라트비아', 'Latvia', '428', 'LV', 'LVA'),
    Country('러시아', 'Russia', '643', 'RU', 'RUS'),
    Country('레바논', 'Lebanon', '422', 'LB', 'LBN'),
    Country('레소토', 'Lesotho', '426', 'LS', 'LSO'),
    Country('레위니옹', 'Réunion', '638', 'RE', 'REU'),
    Country('루마니아', 'Romania', '642', 'RO', 'ROU'),
    Country('룩셈부르크', 'Luxembourg', '442', 'LU', 'LUX'),
    Country('르완다', 'Rwanda', '646', 'RW', 'RWA'),
    Country('리비아', 'Libya', '434', 'LY', 'LBY'),
    Country('리투아니아', 'Lithuania', '440', 'LT', 'LTU'),
    Country('리히텐슈타인', 'Liechtenstein', '438', 'LI', 'LIE'),
    Country('마다가스카르', 'Madagascar', '450', 'MG', 'MDG'),
    Country('마르티니크', 'Martinique', '474', 'MQ', 'MTQ'),
    Country('마셜제도', 'Marshall Islands', '584', 'MH', 'MHL'),
    Country('마요트', 'Mayotte', '175', 'YT', 'MYT'),
    Country('마이크로네시아연방', 'Federated States of Micronesia', '583', 'FM', 'FSM'),
    Country('마카오', 'Macao', '', 'MO', 'MAC'),
    Country('말라위', 'Malawi', '454', 'MW', 'MWI'),
    Country('말레이시아', 'Malaysia', '458', 'MY', 'MYS'),
    Country('말리', 'Mali', '466', 'ML', 'MLI'),
    Country('맨 섬', 'Isle of Man', '833', 'IM', 'IMN'),
    Country('멕시코', 'Mexico', '484', 'MX', 'MEX'),
    Country('모나코', 'Monaco', '492', 'MC', 'MCO'),
    Country('모로코', 'Morocco', '504', 'MA', 'MAR'),
    Country('모리셔스', 'Mauritius', '480', 'MU', 'MUS'),
    Country('모리타니아', 'Mauritania', '478', 'MR', 'MRT'),
    Country('모잠비크', 'Mazambique', '508', 'MZ', 'MOZ'),
    Country('몬테네그로', 'Montenegro', '499', 'ME', 'MNE'),
    Country('몬트세랫', 'Montserrat', '500', 'MS', 'MSR'),
    Country('몰도바', 'Moldova', '498', 'MD', 'MDA'),
    Country('몰디브', 'Maldives', '462', 'MV', 'MDV'),
    Country('몰타', 'Malta', '470', 'MT', 'MLT'),
    Country('몽골', 'Mongolia', '496', 'MN', 'MNG'),
    Country('미얀마', 'Myanmar', '104', 'MM', 'MMR'),
    Country('미합중국', 'United States of America', '840', 'US', 'USA'),
    Country('바누아투', 'Vanuatu', '548', 'VU', 'VUT'),
    Country('바레인', 'Bahrain', '048', 'BH', 'BHR'),
    Country('바베이도스', 'Barbados', '052', 'BB', 'BRB'),
    Country('바하마', 'Bahamas', '044', 'BS', 'BHS'),
    Country('방글라데시', 'Bangladesh', '050', 'BD', 'BGD'),
    Country('버뮤다', 'Bermuda', '060', 'BM', 'BMU'),
    Country('베냉', 'Benin', '204', 'BJ', 'BEN'),
    Country('베네수엘라볼리바르', 'Venezuela', '862', 'VE', 'VEN'),
    Country('베트남', 'Vietnam', '704', 'VN', 'VNM'),
    Country('벨기에', 'Belgium', '056', 'BE', 'BEL'),
    Country('벨라루스', 'Belarus', '112', 'BY', 'BLR'),
    Country('벨리즈', 'Belize', '084', 'BZ', 'BLZ'),
    Country('보스니아헤르체고비나', 'Bosnia-Herzegovina', '070', 'BA', 'BIH'),
    Country('보츠와나', 'Botswana', '072', 'BW', 'BWA'),
    Country('볼리비아', 'Bolivia', '068', 'BO', 'BOL'),
    Country('부룬디', 'Burundi', '108', 'BI', 'BDI'),
    Country('부르키나파소', 'Burkina Faso', '854', 'BF', 'BFA'),
    Country('부탄', 'Bhutan', '064', 'BT', 'BTN'),
    Country('북마리아나 군도', 'Northern Mariana Islands', '580', 'MP', 'MNP'),
    Country('북마케도니아', 'North Macedonia', '807', 'MK', 'MKD'),
    Country('불가리아', 'Bulgaria', '100', 'BG', 'BGR'),
    Country('브라질', 'Brazil', '076', 'BR', 'BRA'),
    Country('브루나이', 'Brunei', '096', 'BN', 'BRN'),
    Country('사모아', 'Samoa', '882', 'WS', 'WSM'),
    Country('사우디아라비아', 'Saudi Arabia', '682', 'SA', 'SAU'),
    Country('사이프러스', 'Cyprus', '196', 'CY', 'CYP'),
    Country('사하라 아랍민주공화국(서부사하라)', 'Sahara Arab Democratic Republic', '732', 'EH', 'ESH'),
    Country('산마리노', 'San Marino', '674', 'SM', 'SMR'),
    Country('상투메프린시페', 'Sâo Tomé & Principe', '678', 'ST', 'STP'),
    Country('생피에르·미클롱', 'St. Pierre and Miquelon', '666', 'PM', 'SPM'),
    Country('세네갈', 'Senegal', '686', 'SN', 'SEN'),
    Country('세르비아', 'Serbia', '688', 'RS', 'SRB'),
    Country('세이셸', 'Seychelles', '690', 'SC', 'SYC'),
    Country('세인트 헬레나 섬', 'St Helena', '654', 'SH', 'SHN'),
    Country('세인트루시아', 'St. Lucia', '662', 'LC', 'LCA'),
    Country('세인트빈센트그레나딘', 'St. Vincent & the Grenadines', '670', 'VC', 'VCT'),
    Country('세인트키츠네비스', 'St. Kitts-Nevis', '659', 'KN', 'KNA'),
    Country('소말리아', 'Somalia', '706', 'SO', 'SOM'),
    Country('솔로몬제도', 'Solomon Islands', '090', 'SB', 'SLB'),
    Country('수단', 'Sudan', '736', 'SD', 'SDN'),
    Country('수리남', 'Suriname', '740', 'SR', 'SUR'),
    Country('스리랑카', 'Sri Lanka', '144', 'LK', 'LKA'),
    Country('스웨덴', 'Sweden', '752', 'SE', 'SWE'),
    Country('스위스', 'Swiss', '756', 'CH', 'CHE'),
    Country('스페인', 'Spain', '724', 'ES', 'ESP'),
    Country('슬로바키아', 'Slovakia', '703', 'SK', 'SVK'),
    Country('슬로베니아', 'Slovenia', '705', 'SI', 'SVN'),
    Country('시리아', 'Syria', '760', 'SY', 'SYR'),
    Country('시에라리온', 'Sierra Leone', '694', 'SL', 'SLE'),
    Country('싱가포르', 'Singapore', '702', 'SG', 'SGP'),
    Country('아랍에미리트', 'United Arab Emirates : UAE', '784', 'AE', 'ARE'),
    Country('아루바', 'Aruba', '533', 'AW', 'ABW'),
    Country('아르메니아', 'Armenia', '051', 'AM', 'ARM'),
    Country('아르헨티나', 'Argentina', '032', 'AR', 'ARG'),
    Country('아세안', '', '   ', 'XA', '   '),
    Country('아이슬란드', 'Iceland', '352', 'IS', 'ISL'),
    Country('아이티', 'Haiti', '332', 'HT', 'HTI'),
    Country('아일랜드', 'Ireland', '372', 'IE', 'IRL'),
    Country('아제르바이잔', 'Azerbaijan', '031', 'AZ', 'AZE'),
    Country('아프가니스탄', 'Afghanistan', '004', 'AF', 'AFG'),
    Country('안도라', 'Andorra', '020', 'AD', 'AND'),
    Country('알바니아', 'Albania', '008', 'AL', 'ALB'),
    Country('알제리', 'Algeria', '012', 'DZ', 'DZA'),
    Country('앙골라', 'Angola', '024', 'AO', 'AGO'),
    Country('앤티가바부다', 'Antigua and Barbuda', '028', 'AG', 'ATG'),
    Country('앵귈라', 'Anguilla', '660', 'AI', 'AIA'),
    Country('에리트레아', 'Eritrea', '232', 'ER', 'ERI'),
    Country('에스와티니', 'Eswatini', '748', 'SZ', 'SWZ'),
    Country('에스토니아', 'Estonia', '233', 'EE', 'EST'),
    Country('에콰도르', 'Ecuador', '218', 'EC', 'ECU'),
    Country('에티오피아', 'ETC', '231', 'ET', 'ETH'),
    Country('엘살바도르', 'El Salvador', '222', 'SV', 'SLV'),
    Country('영국', 'United Kingdom', '826', 'GB', 'GBR'),
    Country('영국령 남극지역', 'British Antarctic Territory', '010', 'AQ', 'ATA'),
    Country('영국령 버진 아일랜드', 'British Virgin Islands', '092', 'VG', 'VGB'),
    Country('영국령 인도양 제도', 'British Indian Ocean Territory', '086', 'IO', 'IOT'),
    Country('예멘', 'Yemen', '887', 'YE', 'YEM'),
    Country('오만', 'Oman', '512', 'OM', 'OMN'),
    Country('오스트리아', 'Austria', '040', 'AT', 'AUT'),
    Country('온두라스', 'Honduras', '340', 'HN', 'HND'),
    Country('왈리스·퓌튀나', 'Wallis and Futuna', '876', 'WF', 'WLF'),
    Country('요르단', 'Jordan', '400', 'JO', 'JOR'),
    Country('우간다', 'Uganda', '800', 'UG', 'UGA'),
    Country('우루과이', 'Uruguay', '858', 'UY', 'URY'),
    Country('우즈베키스탄', 'Uzbekistan', '860', 'UZ', 'UZB'),
    Country('우크라이나', 'Ukraine', '804', 'UA', 'UKR'),
    Country('유네스코', '', '   ', 'XB', '   '),
    Country('유럽연합', 'European Union : EU', '   ', 'EU', '   '),
    Country('이라크', 'Iraq', '368', 'IQ', 'IRQ'),
    Country('이란', 'Iran', '364', 'IR', 'IRN'),
    Country('이스라엘', 'Israel', '376', 'IL', 'ISR'),
    Country('이슬람 국가', '', '   ', 'XC', '   '),
    Country('이집트', 'Egypt', '818', 'EG', 'EGY'),
    Country('이탈리아', 'Italia', '380', 'IT', 'ITA'),
    Country('인도', 'India', '356', 'IN', 'IND'),
    Country('인도네시아', 'Indonesia', '360', 'ID', 'IDN'),
    Country('일본', 'Japan', '392', 'JP', 'JPN'),
    Country('자메이카', 'Jamaica', '388', 'JM', 'JAM'),
    Country('잠비아', 'Zambia', '894', 'ZM', 'ZMB'),
    Country('저지', 'Bailiwick of Jersey', '832', 'JE', 'JEY'),
    Country('적도기니', 'Equatorial Guinea', '226', 'GQ', 'GNQ'),
    Country('전체 국가', '', '', '99', ''),
    Country('조지아', 'Georgia', '268', 'GE', 'GEO'),
    Country('중국', 'China', '156', 'CN', 'CHN'),
    Country('중앙아프리카', 'Central African Republic', '140', 'CF', 'CAF'),
    Country('지부티', 'Djibouti', '262', 'DJ', 'DJI'),
    Country('지브롤터', 'Gibraltar', '292', 'GI', 'GIB'),
    Country('짐바브웨', 'Zimbabwe', '716', 'ZW', 'ZWE'),
    Country('차드', 'Chad', '148', 'TD', 'TCD'),
    Country('체코', 'Czech', '203', 'CZ', 'CZE'),
    Country('칠레', 'Chile', '152', 'CL', 'CHL'),
    Country('카메룬', 'Cameroon', '120', 'CM', 'CMR'),
    Country('카보베르데', 'Cape Verde', '132', 'CV', 'CPV'),
    Country('카자흐스탄', 'Kazakhstan', '398', 'KZ', 'KAZ'),
    Country('카타르', 'Qatar', '634', 'QA', 'QAT'),
    Country('캄보디아', 'Cambodia', '116', 'KH', 'KHM'),
    Country('캐나다', 'Canada', '124', 'CA', 'CAN'),
    Country('케냐', 'Kenya', '404', 'KE', 'KEN'),
    Country('케이맨제도', 'Cayman Islands', '136', 'KY', 'CYM'),
    Country('코모로', 'Comoros', '174', 'KM', 'COM'),
    Country('코소보', 'Kosovo', '383', 'XK', 'XKX'),
    Country('코스타리카', 'Costa Rica', '188', 'CR', 'CRI'),
    Country('코트디부아르', "Côte D'Ivoire", '384', 'CI', 'CIV'),
    Country('콜롬비아', 'Colombia', '170', 'CO', 'COL'),
    Country('콩고', 'Congo', '178', 'CG', 'COG'),
    Country('콩고민주공화국', 'Democratic Republic of Congo', '180', 'CD', 'COD'),
    Country('쿠바', 'Cuba', '192', 'CU', 'CUB'),
    Country('쿠웨이트', 'Kuwait', '414', 'KW', 'KWT'),
    Country('쿡제도', 'Cook Islands', '184', 'CK', 'COK'),
    Country('크로아티아', 'Croatia', '191', 'HR', 'HRV'),
    Country('키르기스스탄', 'Kyrgyz', '417', 'KG', 'KGZ'),
    Country('키리바시', 'Kiribati', '296', 'KI', 'KIR'),
    Country('타지키스탄', 'Tajikistan', '762', 'TJ', 'TJK'),
    Country('탄자니아', 'Tanzania', '834', 'TZ', 'TZA'),
    Country('태국', 'Thailand', '764', 'TH', 'THA'),
    Country('터크스·케이커스 제도', 'Turks and Caicos Islands', '796', 'TC', 'TCA'),
    Country('토고', 'Togo', '768', 'TG', 'TGO'),
    Country('통가', 'Tonga', '776', 'TO', 'TON'),
    Country('투르크메니스탄', 'Turkmenistan', '795', 'TM', 'TKM'),
    Country('투발루', 'Tuvalu', '798', 'TV', 'TUV'),
    Country('튀니지', 'Tunisia', '788', 'TN', 'TUN'),
    Country('튀르키예공화국', 'Republic of Turkiye', '792', 'TR', 'TUR'),
    Country('트리니다드토바고', 'Trinidad & Tobago', '780', 'TT', 'TTO'),
    Country('파나마', 'Panama', '591', 'PA', 'PAN'),
    Country('파라과이', 'Paraguay', '600', 'PY', 'PRY'),
    Country('파키스탄', 'Pakistan', '586', 'PK', 'PAK'),
    Country('파푸아뉴기니', 'Papua New Guinea : PNG', '598', 'PG', 'PNG'),
    Country('팔라우', 'Palau', '585', 'PW', 'PLW'),
    Country('팔레스타인', 'Palestine', '275', 'PS', 'PSE'),
    Country('페루', 'Peru', '604', 'PE', 'PER'),
    Country('포르투갈', 'Portugal', '620', 'PT', 'PRT'),
    Country('폴란드', 'Poland', '616', 'PL', 'POL'),
    Country('푸에르토리코', 'Puerto Rico', '630', 'PR', 'PRI'),
    Country('프랑스', 'France', '250', 'FR', 'FRA'),
    Country('프랑스령 기아나', 'French Guiana', '254', 'GF', 'GUF'),
    Country('프랑스령 폴리네시아', 'French Polynesia', '258', 'PF', 'PYF'),
    Country('피지', 'Fiji', '242', 'FJ', 'FJI'),
    Country('핀란드', 'Finland', '246', 'FI', 'FIN'),
    Country('필리핀', 'Philippines', '608', 'PH', 'PHL'),
    Country('핏케언 섬', 'Pitcairn Islands', '612', 'PN', 'PCN'),
    Country('헝가리', 'Hungary', '348', 'HU', 'HUN'),
    Country('호주', 'Australia', '036', 'AU', 'AUS'),
    Country('홍콩', 'Hongkong', '344', 'HK', 'HKG'),
    Country("가나","GHANA","288","GH","GHA"),
    Country("가봉","GABON","266","GA","GAB"),
    Country("가이아나","GUYANA","328","GY","GUY"),
    Country("감비아","GAMBIA","270","GM","GMB"),
    Country("건지 섬","GUERNSEY","831","GG","GGY"),
    Country("과들루프","GUADELOUPE","312","GP","GLP"),
    Country("과테말라","GUATEMALA","320","GT","GTM"),
    Country("괌","GUAM","316","GU","GUM"),
    Country("그레나다","GRENADA","308","GD","GRD"),
    Country("그리스","GREECE","300","GR","GRC"),
    Country("그린란드","GREENLAND","304","GL","GRL"),
    Country("기니","GUINEA","324","GN","GIN"),
    Country("기니비사우","GUINEA-BISSAU","624","GW","GNB"),
    Country("나미비아","NAMIBIA","516","NA","NAM"),
    Country("나우루","NAURU","520","NR","NRU"),
    Country("나이지리아","NIGERIA","566","NG","NGA"),
    Country("남극","ANTARCTICA","10","AQ","ATA"),
    Country("남수단","REPUBLIC OF SOUTH SUDAN","728","SS","SSD"),
    Country("남아프리카 공화국","SOUTH AFRICA","710","ZA","ZAF"),
    Country("네덜란드","NETHERLANDS","528","NL","NLD"),
    Country("네덜란드령 안틸레스","NETHERLANDS ANTILLES","530","AN","ANT"),
    Country("네팔","NEPAL","524","NP","NPL"),
    Country("노르웨이","NORWAY","578","NO","NOR"),
    Country("노퍽 섬","NORFOLK ISLAND","574","NF","NFK"),
    Country("누벨칼레도니","NEW CALEDONIA","540","NC","NCL"),
    Country("뉴질랜드","NEW ZEALAND","554","NZ","NZL"),
    Country("니우에","NIUE","570","NU","NIU"),
    Country("니제르","NIGER","562","NE","NER"),
    Country("니카라과","NICARAGUA","558","NI","NIC"),
    Country("대한민국","'KOREA","410","KR","KOR"),
    Country("덴마크","DENMARK","208","DK","DNK"),
    Country("도미니카 공화국","DOMINICAN REPUBLIC","214","DO","DOM"),
    Country("도미니카 연방","DOMINICA","212","DM","DMA"),
    Country("독일","GERMANY","276","DE","DEU"),
    Country("동티모르","EAST TIMOR","626","TL","TLS"),
    Country("라오스","LAO PEOPLE'S DEMOCRATIC REPUBLIC","418","LA","LAO"),
    Country("라이베리아","LIBERIA","430","LR","LBR"),
    Country("라트비아","LATVIA","428","LV","LVA"),
    Country("러시아","RUSSIAN FEDERATION","643","RU","RUS"),
    Country("레바논","LEBANON","422","LB","LBN"),
    Country("레소토","LESOTHO","426","LS","LSO"),
    Country("레위니옹","REUNION","638","RE","REU"),
    Country("루마니아","ROMANIA","642","RO","ROU"),
    Country("룩셈부르크","LUXEMBOURG","442","LU","LUX"),
    Country("르완다","RWANDA","646","RW","RWA"),
    Country("리비아","LIBYAN ARAB JAMAHIRIYA","434","LY","LBY"),
    Country("리투아니아","LITHUANIA","440","LT","LTU"),
    Country("리히텐슈타인","LIECHTENSTEIN","438","LI","LIE"),
    Country("마다가스카르","MADAGASCAR","450","MG","MDG"),
    Country("마르티니크","MARTINIQUE","474","MQ","MTQ"),
    Country("마셜 제도","MARSHALL ISLANDS","584","MH","MHL"),
    Country("마요트","MAYOTTE","175","YT","MYT"),
    Country("마카오","MACAU","446","MO","MAC"),
    Country("마케도니아 공화국","REPUBLIC OF MACEDONIA","807","MK","MKD"),
    Country("말라위","MALAWI","454","MW","MWI"),
    Country("말레이시아","MALAYSIA","458","MY","MYS"),
    Country("말리","MALI","466","ML","MLI"),
    Country("맨 섬","ISLE OF MAN","833","IM","IMN"),
    Country("멕시코","MEXICO","484","MX","MEX"),
    Country("모나코","MONACO","492","MC","MCO"),
    Country("모로코","MOROCCO","504","MA","MAR"),
    Country("모리셔스","MAURITIUS","480","MU","MUS"),
    Country("모리타니","MAURITANIA","478","MR","MRT"),
    Country("모잠비크","MOZAMBIQUE","508","MZ","MOZ"),
    Country("몬테네그로","MONTENEGRO","499","ME","MNE"),
    Country("몬트세랫","MONTSERRAT","500","MS","MSR"),
    Country("몰도바","'MOLDOVA","498","MD","MDA"),
    Country("몰디브","MALDIVES","462","MV","MDV"),
    Country("몰타","MALTA","470","MT","MLT"),
    Country("몽골","MONGOLIA","496","MN","MNG"),
    Country("미국","UNITED STATES","840","US","USA"),
    Country("미국령 군소 제도","UNITED STATES MINOR OUTLYING ISLANDS","581","UM","UMI"),
    Country("미국령 버진아일랜드","'VIRGIN ISLANDS","850","VI","VIR"),
    Country("미얀마","MYANMAR","104","MM","MMR"),
    Country("미크로네시아 연방","MICRONESIA","583","FM","FSM"),
    Country("바누아투","VANUATU","548","VU","VUT"),
    Country("바레인","BAHRAIN","48","BH","BHR"),
    Country("바베이도스","BARBADOS","52","BB","BRB"),
    Country("바티칸 시국","VATICAN CITY STATE","336","VA","VAT"),
    Country("바하마","BAHAMAS","44","BS","BHS"),
    Country("방글라데시","BANGLADESH","50","BD","BGD"),
    Country("버뮤다","BERMUDA","60","BM","BMU"),
    Country("베냉","BENIN","204","BJ","BEN"),
    Country("베네수엘라","VENEZUELA","862","VE","VEN"),
    Country("베트남","VIET NAM","704","VN","VNM"),
    Country("벨기에","BELGIUM","56","BE","BEL"),
    Country("벨라루스","BELARUS","112","BY","BLR"),
    Country("벨리즈","BELIZE","84","BZ","BLZ"),
    Country("보스니아 헤르체고비나","BOSNIA HERCEGOVINA","70","BA","BIH"),
    Country("보츠와나","BOTSWANA","72","BW","BWA"),
    Country("볼리비아","BOLIVIA","68","BO","BOL"),
    Country("부룬디","BURUNDI","108","BI","BDI"),
    Country("부르키나파소","BURKINA FASO","854","BF","BFA"),
    Country("부베 섬","BOUVET ISLAND","74","BV","BVT"),
    Country("부탄","BHUTAN","64","BT","BTN"),
    Country("북마리아나 제도","NORTHERN MARIANA ISLANDS","580","MP","MNP"),
    Country("불가리아","BULGARIA","100","BG","BGR"),
    Country("브라질","BRAZIL","76","BR","BRA"),
    Country("브루나이","BRUNEI DARUSSALAM","96","BN","BRN"),
    Country("사모아","SAMOA","882","WS","WSM"),
    Country("사우디아라비아","SAUDI ARABIA","682","SA","SAU"),
    Country("사우스조지아 사우스샌드위치 제도","SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS","239","GS","SGS"),
    Country("산마리노","SAN MARINO","674","SM","SMR"),
    Country("상투메 프린시페","SAO TOME AND PRINCIPE","678","ST","STP"),
    Country("생피에르 미클롱","ST. PIERRE AND MIQUELON","666","PM","SPM"),
    Country("서사하라","WESTERN SAHARA","732","EH","ESH"),
    Country("세네갈","SENEGAL","686","SN","SEN"),
    Country("세르비아","SERBIA","688","RS","SRB"),
    Country("세이셸","SEYCHELLES","690","SC","SYC"),
    Country("세인트루시아","SAINT LUCIA","662","LC","LCA"),
    Country("세인트빈센트 그레나딘","SAINT VINCENT AND THE GRENADINES","670","VC","VCT"),
    Country("세인트키츠 네비스","SAINT KITTS AND NEVIS","659","KN","KNA"),
    Country("세인트헬레나","ST. HELENA","654","SH","SHN"),
    Country("소말리아","SOMALIA","706","SO","SOM"),
    Country("솔로몬 제도","SOLOMON ISLANDS","90","SB","SLB"),
    Country("수단","SUDAN","736","SD","SDN"),
    Country("수리남","SURINAME","740","SR","SUR"),
    Country("스리랑카","SRI LANKA","144","LK","LKA"),
    Country("스발바르 얀마옌","SVALBARD AND JAN MAYEN ISLANDS","744","SJ","SJM"),
    Country("스와질란드","SWAZILAND","748","SZ","SWZ"),
    Country("스웨덴","SWEDEN","752","SE","SWE"),
    Country("스위스","SWITZERLAND","756","CH","CHE"),
    Country("스페인","SPAIN","724","ES","ESP"),
    Country("슬로바키아","SLOVAKIA","703","SK","SVK"),
    Country("슬로베니아","SLOVENIA","705","SI","SVN"),
    Country("시리아","SYRIAN ARAB REPUBLIC","760","SY","SYR"),
    Country("시에라리온","SIERRA LEONE","694","SL","SLE"),
    Country("싱가포르","SINGAPORE","702","SG","SGP"),
    Country("아랍에미리트","UNITED ARAB EMIRATES","784","AE","ARE"),
    Country("아루바","ARUBA","533","AW","ABW"),
    Country("아르메니아","ARMENIA","51","AM","ARM"),
    Country("아르헨티나","ARGENTINA","32","AR","ARG"),
    Country("아메리칸사모아","AMERICAN SAMOA","16","AS","ASM"),
    Country("아이슬란드","ICELAND","352","IS","ISL"),
    Country("아이티","HAITI","332","HT","HTI"),
    Country("아일랜드","IRELAND","372","IE","IRL"),
    Country("아제르바이잔","AZERBAIJAN","31","AZ","AZE"),
    Country("아프가니스탄","AFGHANISTAN","4","AF","AFG"),
    Country("안도라","ANDORRA","20","AD","AND"),
    Country("알바니아","ALBANIA","8","AL","ALB"),
    Country("알제리","ALGERIA","12","DZ","DZA"),
    Country("앙골라","ANGOLA","24","AO","AGO"),
    Country("앤티가 바부다","ANTIGUA AND BARBUDA","28","AG","ATG"),
    Country("앵귈라","ANGUILLA","660","AI","AIA"),
    Country("에리트레아","ERITREA","232","ER","ERI"),
    Country("에스토니아","ESTONIA","233","EE","EST"),
    Country("에콰도르","ECUADOR","218","EC","ECU"),
    Country("에티오피아","ETHIOPIA","231","ET","ETH"),
    Country("엘살바도르","EL SALVADOR","222","SV","SLV"),
    Country("영국","UNITED KINGDOM","826","GB","GBR"),
    Country("영국령 버진아일랜드","'VIRGIN ISLANDS","92","VG","VGB"),
    Country("영국령 인도양 지역","BRITISH INDIAN OCEAN TERRITORY","86","IO","IOT"),
    Country("예멘","'YEMEN","887","YE","YEM"),
    Country("오만","OMAN","512","OM","OMN"),
    Country("오스트레일리아","AUSTRALIA","36","AU","AUS"),
    Country("오스트리아","AUSTRIA","40","AT","AUT"),
    Country("온두라스","HONDURAS","340","HN","HND"),
    Country("올란드 제도","ALAND ISLANDS","248","AX","ALA"),
    Country("왈리스 퓌튀나","WALLIS AND FUTUNA ISLANDS","876","WF","WLF"),
    Country("요르단","JORDAN","400","JO","JOR"),
    Country("우간다","UGANDA","800","UG","UGA"),
    Country("우루과이","URUGUAY","858","UY","URY"),
    Country("우즈베키스탄","UZBEKISTAN","860","UZ","UZB"),
    Country("우크라이나","UKRAINE","804","UA","UKR"),
    Country("이라크","IRAQ","368","IQ","IRQ"),
    Country("이란","IRAN","364","IR","IRN"),
    Country("이스라엘","ISRAEL","376","IL","ISR"),
    Country("이집트","EGYPT","818","EG","EGY"),
    Country("이탈리아","ITALY","380","IT","ITA"),
    Country("인도","INDIA","356","IN","IND"),
    Country("인도네시아","INDONESIA","360","ID","IDN"),
    Country("일본","JAPAN","392","JP","JPN"),
    Country("자메이카","JAMAICA","388","JM","JAM"),
    Country("잠비아","ZAMBIA","894","ZM","ZMB"),
    Country("저지 섬","JERSEY","832","JE","JEY"),
    Country("적도 기니","EQUATORIAL GUINEA","226","GQ","GNQ"),
    Country("조선민주주의인민공화국","'KOREA","408","KP","PRK"),
    Country("조지아","GEORGIA","268","GE","GEO"),
    Country("중앙아프리카 공화국","CENTRAL AFRICAN REPUBLIC","140","CF","CAF"),
    Country("중화민국","'TAIWAN","158","TW","TWN"),
    Country("중화인민공화국","CHINA","156","CN","CHN"),
    Country("지부티","DJIBOUTI","262","DJ","DJI"),
    Country("지브롤터","GIBRALTAR","292","GI","GIB"),
    Country("짐바브웨","ZIMBABWE","716","ZW","ZWE"),
    Country("차드","CHAD","148","TD","TCD"),
    Country("체코","CZECH REPUBLIC","203","CZ","CZE"),
    Country("칠레","CHILE","152","CL","CHL"),
    Country("카메룬","CAMEROON","120","CM","CMR"),
    Country("카보베르데","CAPE VERDE","132","CV","CPV"),
    Country("카자흐스탄","KAZAKHSTAN","398","KZ","KAZ"),
    Country("카타르","QATAR","634","QA","QAT"),
    Country("캄보디아","CAMBODIA","116","KH","KHM"),
    Country("캐나다","CANADA","124","CA","CAN"),
    Country("케냐","KENYA","404","KE","KEN"),
    Country("케이맨 제도","CAYMAN ISLANDS","136","KY","CYM"),
    Country("코모로","COMOROS","174","KM","COM"),
    Country("코스타리카","COSTA RICA","188","CR","CRI"),
    Country("코코스 제도","COCOS ISLANDS","166","CC","CCK"),
    Country("코트디부아르","COTE D'IVOIRE","384","CI","CIV"),
    Country("콜롬비아","COLOMBIA","170","CO","COL"),
    Country("콩고 공화국","CONGO","178","CG","COG"),
    Country("콩고 민주 공화국","DEMOCRATIC REPUBLIC OF THE CONGO","180","CD","COD"),
    Country("쿠바","CUBA","192","CU","CUB"),
    Country("쿠웨이트","KUWAIT","414","KW","KWT"),
    Country("쿡 제도","COOK ISLANDS","184","CK","COK"),
    Country("크로아티아","CROATIA","191","HR","HRV"),
    Country("크리스마스 섬","CHRISTMAS ISLAND","162","CX","CXR"),
    Country("키르기스스탄","KYRGYZSTAN","417","KG","KGZ"),
    Country("키리바시","KIRIBATI","296","KI","KIR"),
    Country("키프로스","CYPRUS","196","CY","CYP"),
    Country("타이","THAILAND","764","TH","THA"),
    Country("타지키스탄","TAJIKISTAN","762","TJ","TJK"),
    Country("탄자니아","'TANZANIA","834","TZ","TZA"),
    Country("터크스 케이커스 제도","TURKS AND CAICOS ISLANDS","796","TC","TCA"),
    Country("터키","TURKEY","792","TR","TUR"),
    Country("토고","TOGO","768","TG","TGO"),
    Country("토켈라우","TOKELAU","772","TK","TKL"),
    Country("통가","TONGA","776","TO","TON"),
    Country("투르크메니스탄","TURKMENISTAN","795","TM","TKM"),
    Country("투발루","TUVALU","798","TV","TUV"),
    Country("튀니지","TUNISIA","788","TN","TUN"),
    Country("트리니다드 토바고","TRINIDAD AND TOBAGO","780","TT","TTO"),
    Country("파나마","PANAMA","591","PA","PAN"),
    Country("파라과이","PARAGUAY","600","PY","PRY"),
    Country("파키스탄","PAKISTAN","586","PK","PAK"),
    Country("파푸아 뉴기니","PAPUA NEW GUINEA","598","PG","PNG"),
    Country("팔라우","PALAU","585","PW","PLW"),
    Country("팔레스타인","PALESTINE","275","PS","PSE"),
    Country("페로 제도","FAROE ISLANDS","234","FO","FRO"),
    Country("페루","PERU","604","PE","PER"),
    Country("포르투갈","PORTUGAL","620","PT","PRT"),
    Country("포클랜드 제도","FALKLAND ISLANDS","238","FK","FLK"),
    Country("폴란드","POLAND","616","PL","POL"),
    Country("푸에르토리코","PUERTO RICO","630","PR","PRI"),
    Country("프랑스","FRANCE","250","FR","FRA"),
    Country("프랑스령 기아나","FRENCH GUIANA","254","GF","GUF"),
    Country("프랑스령 남부와 남극 지역","FRENCH SOUTHERN TERRITORIES","260","TF","ATF"),
    Country("프랑스령 폴리네시아","FRENCH POLYNESIA","258","PF","PYF"),
    Country("피지","FIJI","242","FJ","FJI"),
    Country("핀란드","FINLAND","246","FI","FIN"),
    Country("필리핀","PHILIPPINES","608","PH","PHL"),
    Country("핏케언 제도","PITCAIRN","612","PN","PCN"),
    Country("허드 맥도널드 제도","HEARD AND MC DONALD ISLANDS","334","HM","HMD"),
    Country("헝가리","HUNGARY","348","HU","HUN"),
    Country("홍콩","HONG KONG","344","HK","HKG"),

]


def _build_index(idx: int) -> Dict[str, Country]:
    return dict((r[idx].upper(), r) for r in _records)


# Internal country indexes
_by_alpha2 = _build_index(3)
_by_alpha3 = _build_index(4)
_by_numeric = _build_index(2)
_by_name = _build_index(0)
_by_apolitical_name = _build_index(1)


# Documented accessors for the country indexes
countries_by_alpha2 = _by_alpha2
countries_by_alpha3 = _by_alpha3
countries_by_numeric = _by_numeric
countries_by_name = _by_name
countries_by_apolitical_name = _by_apolitical_name


class NotFound:
    pass


class _CountryLookup:
    @overload
    def get(self, key: StrOrInt) -> Country:
        ...

    @overload
    def get(self, key: StrOrInt, default: _D) -> Union[Country, _D]:
        ...

    def get(
        self, key: StrOrInt, default: Union[Type[NotFound], _D] = NotFound
    ) -> Union[Country, _D]:
        if isinstance(key, int):
            k = f"{key:03d}"
            r = _by_numeric.get(k, default)
        else:
            k = key
            # if len(k) == 2:
            #     r = _by_alpha2.get(k, default)
            # elif len(k) == 3 and re.match(r"[0-9]{3}", k) and k != "000":
            #     r = _by_numeric.get(k, default)
            # elif len(k) == 3:
            #     r = _by_alpha3.get(k, default)
            # elif k in _by_name:
            r = _by_name.get(k, default)
            # else:
            # r = _by_apolitical_name.get(k, default)

        if r == NotFound:
            raise KeyError(key)

        return r

    __getitem__ = get

    def __len__(self) -> int:
        return len(_records)

    def __iter__(self) -> Iterator[Country]:
        return iter(_records)

    def __contains__(self, item: StrOrInt) -> bool:
        try:
            self.get(item)
            return True
        except KeyError:
            return False


countries = _CountryLookup()